#include "gtest/gtest.h"
#include "../Trie.h"
#include <iostream>
#include <string>
#include <vector>
using namespace std;

class Testing : public ::testing::Test {

protected:
    Testing() {}
    virtual ~Testing() {
    }

    virtual void SetUp() {}
    
    virtual void TearDown() {
	cout << endl << ":::END" << endl;
    }
};
    

TEST_F(Testing, isAWordBasic) {
    cout << ":::NAME test isAWord basic" << endl;
    cout << ":::MAX_SCORE 6" << endl;
    cout << ":::SCORE 6" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    cout << "Calling isAWord('cat'):" << endl;
    EXPECT_TRUE(searchDict.isAWord("cat")) << ":::SCORE 0";
    cout << "Calling isAWord('cast'):" << endl;
    EXPECT_TRUE(searchDict.isAWord("cast")) << ":::SCORE 0";
    cout << "Calling isAWord('dog'):" << endl;
    EXPECT_TRUE(searchDict.isAWord("dog")) << ":::SCORE 0";
}


TEST_F(Testing, isAWordIsWordIsNotAWord) {
    cout << ":::NAME test isAWord query is not a word" << endl;
    cout << ":::MAX_SCORE 6" << endl;
    cout << ":::SCORE 6" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    cout << "Calling isAWord('ca'):" << endl;
    EXPECT_FALSE(searchDict.isAWord("ca")) << ":::SCORE 0";
    cout << "Calling isAWord('x'):" << endl;
    EXPECT_FALSE(searchDict.isAWord("x")) << ":::SCORE 0";
}

TEST_F(Testing, isAWordIsWordEmptyString) {
    cout << ":::NAME test isAWord empty string" << endl;
    cout << ":::MAX_SCORE 3" << endl;
    cout << ":::SCORE 3" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    EXPECT_FALSE(searchDict.isAWord("")) << ":::SCORE 0";
}


TEST_F(Testing, prefixbasic) {
    cout << ":::NAME test allWordsBeginningWithPrefix basic" << endl;
    cout << ":::MAX_SCORE 9" << endl;
    cout << ":::SCORE 9" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    cout << "Calling allWordsBeginningWithPrefix('ca'):" << endl;
    std::vector<string> words = searchDict.allWordsBeginningWithPrefix("ca");
    EXPECT_EQ(words.size(), 2) << ":::SCORE 0";
    bool found = false;
    for (string word : words)
	if (word == "cat") found = true;
    EXPECT_TRUE(found) << ":::SCORE 0";
    found = false;
    for (string word : words)
	if (word == "cast") found = true;
    EXPECT_TRUE(found) << ":::SCORE 0";
}

TEST_F(Testing, prefixIsWord) {
    cout << ":::NAME test allWordsBeginningWithPrefix prefix is word" << endl;
    cout << ":::MAX_SCORE 3" << endl;
    cout << ":::SCORE 3" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    cout << "Calling allWordsBeginningWithPrefix('cat'):" << endl;
    std::vector<string> words = searchDict.allWordsBeginningWithPrefix("cat");
    if (words.size() == 0) {
	cout << "Failed to find 'cat' as a word beginning with 'cat'" << endl;
	EXPECT_EQ(words.size(), 1) << ":::SCORE 0";
    }
    else 
	EXPECT_EQ(words[0], "cat") << ":::SCORE 0";    
}



TEST_F(Testing, prefixNotInTree) {
    cout << ":::NAME test allWordsBeginningWithPrefix where prefix is not in the Trie" << endl;
    cout << ":::MAX_SCORE 3" << endl;
    cout << ":::SCORE 3" << endl;
    cout << ":::VISIBILITY visible" << endl;
    cout << "\nTesting with a Trie with cat, cast, dog added." << endl;
    Trie searchDict;
    searchDict.addAWord("cat");
    searchDict.addAWord("cast");
    searchDict.addAWord("dog");
		       
    cout << "Calling allWordsBeginningWithPrefix('x'):" << endl;
    std::vector<string> words = searchDict.allWordsBeginningWithPrefix("x");
    EXPECT_EQ(words.size(), 0) << ":::SCORE 0";
    
}

int main(int argc, char **argv) {
  testing::InitGoogleTest(&argc, argv);
  RUN_ALL_TESTS();
  return 0;
}
